#!/bin/bash
#
# ===== pve-host scripts ======
#
# snap-create.sh: snapshot creation.
#
# $Rev: 81 $
# $Date $
# $Author: resolir $
#
#	Copyright (C) 2011-2012 Roberto Resoli - Comune di Trento;
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; version 3
#	of the License.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
#	MA  02111-1307, USA.
#

source /usr/share/backuppc-bcs/bin/snap-utils

if [ $# -lt 8 ]; then exit 1; fi

VMID=$1
DISK=$2
PART=p$3
PART_TYPE=$4
PART_MOUNT=$5
STORAGE=$6
MBR_SAVE=$7
SNAP_SIZE=$8
MP_PREFIX=$9

if [ "$MP_PREFIX" != "" ]; then
        MP_PREFIX="/$MP_PREFIX"
fi

MOUNT_OK=999
PART_MOUNT_POINT="/tmp/$VMID$MP_PREFIX$PART_MOUNT"
PART_LOCK_FILE=/tmp/$VMID-bkp-$STORAGE-disk$DISK$PART.lock

check_create_mountpoint(){
  local mountpoint=$1
  if [ ! -d "$mountpoint" ]; then
    mkdir -p "$mountpoint"
  fi      
}

if [ -f $PART_LOCK_FILE ]; then
  echo "$VMID: Partition $PART on disk $DISK on storage $STORAGE is in use"
  exit 1
fi

VOLUME=vm-$VMID-disk-$DISK

if [ "$STORAGE" == "local" ]; then
  VG=pve
  LV=data
else
  VG=$STORAGE
  LV=$VOLUME
fi

check_create_mountpoint "$PART_MOUNT_POINT"

LV_SNAP=snap-$VG-$LV
LV_MOUNT_POINT=/tmp/bkp-$VG-$LV

if [[ $SNAP_SIZE == *FREE*  ]]; then  
  SNAP_SIZE_FLAG="-l"
else
  SNAP_SIZE_FLAG="-L"
fi  


retry_on_fail "snap_create $LV_SNAP /dev/$VG/$LV $SNAP_SIZE_FLAG $SNAP_SIZE $LV_MOUNT_POINT.lock" 3 10 
if [ $? -ne 0  ]; then
	echo "Snap creation failed! Aborting."
	exit 1
fi

#if [ ! -f $LV_MOUNT_POINT.lock ]; then
# touch $LV_MOUNT_POINT.lock
# 
# echo "snap command: lvcreate -s -n $LV_SNAP /dev/$VG/$LV $SNAP_SIZE_FLAG $SNAP_SIZE"
#  lvcreate -s -n $LV_SNAP /dev/$VG/$LV $SNAP_SIZE_FLAG $SNAP_SIZE
#  if [ $? -ne 0  ]; then
#    echo "Error occured making snapshot!"
#    rm $LV_MOUNT_POINT.lock
#    exit 1
#  fi
#else
#  echo "$LV_MOUNT_POINT.lock is present! Aborting."
#  exit 1
#fi

#wait few seconds before running kpartx
echo sleeping 5 seconds ...
sleep 5

if [ "$STORAGE" == "local" ]; then
  check_create_mountpoint "$LV_MOUNT_POINT"

  mountpoint -q $LV_MOUNT_POINT
  if [ $? -ne 0 ]; then
    mount /dev/$VG/$LV_SNAP $LV_MOUNT_POINT
  fi

  LV_DEV=`losetup -f`
  losetup $LV_DEV $LV_MOUNT_POINT/images/$VMID/$VOLUME.raw

  kpartx -av $LV_DEV
  DEV=${LV_DEV/loop/mapper/loop}$PART
else
  LV_DEV=/dev/$VG/$LV_SNAP
  LV_SNAP_DEV_MAPPER=/dev/mapper/$VG-${LV_SNAP//-/--}
  DEV=$LV_SNAP_DEV_MAPPER$PART
  kpartx -av $LV_DEV
fi


if [ "$MBR_SAVE" == "mbryes" -o "$PART_TYPE" == "ntfs" ]; then
  echo "Mount snapshot rw in \"$PART_MOUNT_POINT\" " 
  if [ "$PART_TYPE" == "ntfs" ]; then
	mount -t $PART_TYPE-3g -o rw,force,nls=utf8 $DEV "$PART_MOUNT_POINT"
  else
	mount -t $PART_TYPE -o rw $DEV "$PART_MOUNT_POINT"
  fi
  MOUNT_OK=$?
  if [ $MOUNT_OK -eq 0 ]; then
     if [ "$PART_TYPE" == "ntfs" ]; then
	#ROB: Creazione directory per mappatura attributi ntfs in attributi estesi e salvataggio degli stessi.
	#N.B.: Solo per ntfs-3g compilato dai sorgenti; pacchetto ntfs-3g di lenny troppo vecchio per questo. 
	rsync -a --exclude ".svn" /usr/share/backuppc-bcs/bin/.NTFS-3G "$PART_MOUNT_POINT"/ > /dev/null 2>&1 
	echo "NTFS-3G dir created in  \"$PART_MOUNT_POINT\" " 
   	echo "Remount read-only using ntfs-3g 'streams_interface=xattr' option." 
  	umount "$PART_MOUNT_POINT"
	mount -t $PART_TYPE-3g -o ro,force,nls=utf8,streams_interface=xattr $DEV "$PART_MOUNT_POINT"
	ATTR_BAK=".attr.bak"
	ATTR_ERR=".attr.err"
	ATTR_TMP=${PART_LOCK_FILE%.lock}
	echo -n "Saving extended attributes ... "
	cd "$PART_MOUNT_POINT"
	#Complicated, but it works: compress on the fly stderr and stdout
	#to distinct files.
	{
	  {
    	     getfattr -R -d . 3>&- | gzip -c > "$ATTR_TMP""$ATTR_BAK".gz 2>&3 3>&-
  	  } 2>&1 >&4 4>&- | gzip -c > "$ATTR_TMP""$ATTR_ERR".gz 3>&- 4>&-

	} 3>&2 4>&1
	echo "done."
	cd /tmp
   	echo -n "Remounting read-write ... " 
  	umount "$PART_MOUNT_POINT"
	mount -t $PART_TYPE-3g -o rw,force,nls=utf8 $DEV "$PART_MOUNT_POINT"
	echo "done."
	
	echo -n "Saving ntfs attributes to \"${PART_MOUNT_POINT/%\//}/$ATTR_BAK.gz\" ... " 

	mv "$ATTR_TMP""$ATTR_BAK".gz "${PART_MOUNT_POINT/%\//}"/"$ATTR_BAK".gz
	mv "$ATTR_TMP""$ATTR_ERR".gz "${PART_MOUNT_POINT/%\//}"/"$ATTR_ERR".gz
	echo "done."
       
	echo -n "Saving partition boot sector to \"${PART_MOUNT_POINT/%\//}/.pbs.bak\" ... " 
	dd if=$DEV bs=512 count=1 of="${PART_MOUNT_POINT/%\//}/.pbs.bak" > /dev/null 2>&1 
	echo "done."
     fi
     #These operations are filesystem independent.
     echo -n "Saving master boot record to \"${PART_MOUNT_POINT/%\//}/.mbr.bak\" ... " 
     dd if=$LV_DEV bs=512 count=1 of="${PART_MOUNT_POINT/%\//}/.mbr.bak" > /dev/null 2>&1 
     echo "done."
	
     echo -n "Saving master boot record info to \"${PART_MOUNT_POINT/%\//}/.mbr.info\" ... " 
     fdisk -lu $LV_DEV > "${PART_MOUNT_POINT/%\//}/.mbr.info"
     echo "done."
  fi

  echo -n "Unmount ... "
  umount "$PART_MOUNT_POINT"
  if [ $? -eq 0 ]; then
    echo "done"
  fi
fi

echo -n "Mount readonly ... "
if [ "$PART_TYPE" == "ntfs" ]; then
  mount -t $PART_TYPE-3g -o ro,nls=utf8 $DEV "$PART_MOUNT_POINT"
else
  mount -t $PART_TYPE -o ro $DEV "$PART_MOUNT_POINT"
fi
MOUNT_OK=$?
if [ $MOUNT_OK -eq 0 ]; then
  echo "done"
fi


if [ $MOUNT_OK -eq 0 ]; then
#  if [ "$STORAGE" == "local" ]; then
    ln $LV_MOUNT_POINT.lock $PART_LOCK_FILE
#  else
#    touch $PART_LOCK_FILE
#  fi
fi

exit $MOUNT_OK
