#!/bin/bash
#
# ===== pve-host scripts ======
#
# snap-remove.sh: snapshot removal
#
# $Rev: 81 $
# $Date $
# $Author: resolir $
#
#	Copyright (C) 2011-2012 Roberto Resoli - Comune di Trento;
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; version 3
#	of the License.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
#	MA  02111-1307, USA.
#

source /usr/share/backuppc-bcs/bin/snap-utils

if [ $# -lt 8 ]; then exit 1; fi
VMID=$1
DISK=$2
PART=p$3
PART_MOUNT=$5
STORAGE=$6
MP_PREFIX=$9

if [ "$MP_PREFIX" != "" ]; then
        MP_PREFIX="/$MP_PREFIX"
fi

PART_MOUNT_POINT="/tmp/$VMID$MP_PREFIX$PART_MOUNT"

VOLUME=vm-$VMID-disk-$DISK
PART_LOCK_FILE=/tmp/$VMID-bkp-$STORAGE-disk$DISK$PART.lock

echo Storage: $STORAGE

if [ "$STORAGE" == "local" ]; then
  VG=pve
  LV=data
  LPDEV=`losetup -a | grep $VMID-disk-$DISK | awk '{print $1}'`
  LV_DEV=${LPDEV%:}
else
  VG=$STORAGE
  LV=$VOLUME
  LV_DEV=/dev/$VG/snap-$VG-$LV
fi


LV_MOUNT_POINT=/tmp/bkp-$VG-$LV

mountpoint -q $PART_MOUNT_POINT
if [ $? -eq 0 ]; then
  echo "Unmount \"$PART_MOUNT_POINT\" ..."
  umount $PART_MOUNT_POINT
  if [ $? -eq 0 ]; then
    rm $PART_LOCK_FILE
  else
    echo "Error unmounting $PART_MOUNT_POINT"
    exit 1
  fi
else
  echo "$PART_MOUNT_POINT not mounted"
  exit 0
fi


if [ -f /tmp/bkp-$VG-$LV.lock ]; then
  NLOCKS=`ls -l /tmp/bkp-$VG-$LV.lock | awk '{ print $2 }'`
else
  NLOCKS=0
fi

if [ $NLOCKS -le 1 ]; then

  #ROB: Commented out; delaying snapshot removal cold cause 
  #	a race condition with an incoming backuppc dump
  #	on another partition on same volume.
  #wait few seconds before running kpartx
  #echo sleeping 5 seconds ...
  #sleep 5

  echo "Remove partitions mappings ..."
  retry_on_fail "kpartx -dv $LV_DEV" 3 5
  if [ $? -ne 0 ]; then
    echo "Can't remove mappings! Aborting."
    exit 1
  fi

  if [ "$STORAGE" == "local" ]; then
    losetup -d $LV_DEV
  fi

  if [ "$STORAGE" == "local" ]; then
    umount $LV_MOUNT_POINT 
  fi

  rm /tmp/bkp-$VG-$LV.lock 

  echo "Remove snapshot ..."
  retry_on_fail "lvremove -f $VG/snap-$VG-$LV" 3 5
  if [ $? -ne 0 ]; then
    echo "Can't remove snapshot: maybe it is full? Aborting."
    exit 1
  fi

fi

exit 0
